subroutine calcin(option, max, may, maz, model,&
                  veprj, modx, mody, modz, i,&
                  j, mij)
!-------------------------------------------------------------------
! ======================================================================
! COPYRIGHT (C) 1991 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
!-------------------------------------------------------------------
    implicit none
!
! ROUTINE CALCULANT LA MASSE AJOUTEE SUR LE MODELE THERMIQUE
!  D INTERFACE AINSI QUE LE PREMIER COEFFICIENT D AMORTISSEMENT
!  AJOUTE
! IN :MAX,MAY,MAZ : MATRICES AX ET AY ET AZ
! IN: VEPRJ: PRESSION PROJETEE DUE AU MODE OU CHAMNO J
! IN: MODEL: K2 : CHAINE DISTINGUANT LE TYPE DE MODELISATION
! IN: MODX,MODY,MODZ : DEPLACEMENTS PROJETES
! OUT : MIJ : MASSE AJOUTEE
!-------------------------------------------------------------------
#include "jeveux.h"
#include "asterfort/detrsd.h"
#include "asterfort/jedema.h"
#include "asterfort/jedetr.h"
#include "asterfort/jelira.h"
#include "asterfort/jemarq.h"
#include "asterfort/jeveuo.h"
#include "asterfort/mrmult.h"
#include "asterfort/mtdscr.h"
#include "asterfort/utmess.h"
#include "asterfort/wkvect.h"
#include "asterfort/as_deallocate.h"
#include "asterfort/as_allocate.h"
#include "blas/ddot.h"
    integer ::  i, j
    real(kind=8) :: mij
    character(len=*) :: model, option
    character(len=19) :: modx, mody, modz, veprj, max, may, maz
!--------------------------------------------------------------------
!
!-----------------------------------------------------------------------
    integer :: imatx, imaty, imatz
    integer ::   nbpres
    real(kind=8) :: rx, ry, rz
    real(kind=8), pointer :: vectx(:) => null()
    real(kind=8), pointer :: vecty(:) => null()
    real(kind=8), pointer :: vectz(:) => null()
    real(kind=8), pointer :: vmodx(:) => null()
    real(kind=8), pointer :: vmody(:) => null()
    real(kind=8), pointer :: vmodz(:) => null()
    real(kind=8), pointer :: pres(:) => null()
!-----------------------------------------------------------------------
    call jemarq()
    call jeveuo(modx//'.VALE', 'L', vr=vmodx)
    call jeveuo(mody//'.VALE', 'L', vr=vmody)
!
    call jeveuo(veprj//'.VALE', 'L', vr=pres)
    call jelira(veprj//'.VALE', 'LONMAX', nbpres)
!
    AS_ALLOCATE(vr=vectx, size=nbpres)
    AS_ALLOCATE(vr=vecty, size=nbpres)
!
! --- RECUPERATION DES DESCRIPTEURS DE MATRICES ASSEMBLEES MAX ET MAY
!
    call mtdscr(max)
    call jeveuo(max(1:19)//'.&INT', 'E', imatx)
    call mtdscr(may)
    call jeveuo(may(1:19)//'.&INT', 'E', imaty)
!
!------MULTIPLICATIONS MATRICE MAX * CHAMNO MODX---------------------
!----------ET MATRICE MAY * CHAMNO MODY------------------------------
!
    call mrmult('ZERO', imatx, vmodx, vectx, 1,&
                .true._1)
    call mrmult('ZERO', imaty, vmody, vecty, 1,&
                .true._1)
!
!--PRODUITS SCALAIRES VECTEURS PRESSION PAR MAX*MODX ET MAY*MODY
!
    rx= ddot(nbpres,pres, 1,vectx,1)
    ry= ddot(nbpres,pres, 1,vecty,1)
!
!
!---------------- MENAGE SUR LA VOLATILE ---------------------------
!
!
    AS_DEALLOCATE(vr=vectx)
    AS_DEALLOCATE(vr=vecty)
!
    call detrsd('CHAM_NO', modx)
    call detrsd('CHAM_NO', mody)
!
!
!
!
!
!------ MASSE AJOUTEE = PRESSION*MAX*MODX + PRESSION*MAY*MODY-------
!--------------------------+ PRESSION*MAZ*MODZ  EN 3D---------------
    if (model .eq. '3D') then
!
        call jeveuo(modz//'.VALE', 'L', vr=vmodz)
        AS_ALLOCATE(vr=vectz, size=nbpres)
        call mtdscr(maz)
        call jeveuo(maz(1:19)//'.&INT', 'E', imatz)
        call mrmult('ZERO', imatz, vmodz, vectz, 1,&
                    .true._1)
        rz= ddot(nbpres,pres, 1,vectz,1)
        AS_DEALLOCATE(vr=vectz)
        call detrsd('CHAM_NO', modz)
        mij = rx+ry+rz
!
    else
        mij = rx+ry
    endif
!
    if ((i.eq.j) .and. (mij.lt.0) .and. (option.eq.'MASS_AJOU')) then
        call utmess('A', 'ALGORITH_60')
    endif
    if ((i.eq.j) .and. (mij.lt.0) .and. (option.eq.'AMOR_AJOU')) then
        call utmess('A', 'ALGORITH_61')
    endif
!
    call detrsd('CHAM_NO', veprj)
!
!-----------------------------------------------------------------
    call jedema()
end subroutine
