subroutine xstam1(noma, nbma, nmafis, mafis,&
                  stano, mafon, maen1, maen2, maen3,&
                  nmafon, nmaen1, nmaen2, nmaen3,&
                  typdis, cnslt)
!
! ======================================================================
! COPYRIGHT (C) 1991 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
! person_in_charge: samuel.geniaut at edf.fr
!
    implicit none
#include "jeveux.h"
!
#include "asterfort/assert.h"
#include "asterfort/dismoi.h"
#include "asterfort/jedema.h"
#include "asterfort/jemarq.h"
#include "asterfort/jenuno.h"
#include "asterfort/jeveuo.h"
#include "asterfort/jexatr.h"
#include "asterfort/jexnum.h"
#include "asterfort/panbno.h"
    integer :: nmafis, nmafon, nmaen1, nmaen2, nmaen3, nbma
    integer :: stano(*), mafis(nmafis)
    integer :: mafon(nmafis), maen1(nbma), maen2(nbma), maen3(nbma)
    character(len=8) :: noma
    character(len=16) :: typdis
    character(len=19) :: cnslt
!
! ----------------------------------------------------------------------
!
! ROUTINE XFEM
!
! CALCUL DU STATUT DES MAILLES SUIVANT LE STATUT DES NOEUDS EN COURS
!
! ----------------------------------------------------------------------
!
! IN  NOMA   : NOM DU MAILLAGE
! IN  NBMA   : NOMBRE DE MAILLES DU MAILLAGE
! IN  NMAFIS : NOMBRE DE MAILLES DE LA ZONE FISSURE
! IN  MAFIS  : VECTUER DES MAILLES DE LA ZONE FISSURE
! IN  STANO  : VECTEUR STATUT DES NOEUDS
!
! OUT  NMAFON : NOMBRE DE MAILLES CONTENANT LE FOND DE FISSURE
! OUT  NMAEN1 : NOMBRE DE MAILLES 'HEAVISIDE'
! OUT  NMAEN2 : NOMBRE DE MAILLES 'CRACKTIP'
! OUT  NMAEN3 : NOMBRE DE MAILLES 'HEAVISIDE-CRACKTIP'
! OUT  MAFON  : VECTEUR DES MAILLES 'CONTENANT LE FOND DE FISSURE
! OUT  MAEN1  : VECTEUR DES MAILLES 'HEAVISIDE'
! OUT  MAEN2  : VECTEUR DES MAILLES 'CRACKTIP'
! OUT  MAEN3  : VECTEUR DES MAILLES 'HEAVISIDE-CRACKTIP'
!
!
!
!
    integer ::  jma, nuno,  jconx2, jltsv
    integer :: i, im1, im2, im3, ima, itypma, in, imae, nunop
    integer :: em, em1, em2, nmaabs, nbnott(3), nno, en
    integer :: ndim, dime_topo
    character(len=8) :: typma
    character(len=19) :: mai
    aster_logical :: lstch
    integer, pointer :: connex(:) => null()
!
    call jemarq()
!
    i=0
    im1=0
    im2=0
    im3=0
!
    mai=noma//'.TYPMAIL'
    call jeveuo(mai, 'L', jma)
    call jeveuo(noma//'.CONNEX', 'L', vi=connex)
    call jeveuo(jexatr(noma//'.CONNEX', 'LONCUM'), 'L', jconx2)
    if(cnslt(3:8).eq.'OP0010') call jeveuo(cnslt//'.CNSV','L',jltsv)
!
!   dimension du maillage
    call dismoi('DIM_GEOM', noma, 'MAILLAGE', repi=ndim)
    ASSERT(ndim .eq. 2 .or. ndim .eq. 3)
!
!     BOUCLE SUR LES MAILLES DU MAILLAGE
    do 310 ima = 1, nbma
!
        nmaabs=ima
!
        itypma=zi(jma-1+nmaabs)
        call jenuno(jexnum('&CATA.TM.NOMTM', itypma), typma)
        if (typma(1:3) .eq. 'POI') goto 310
!
        em=0
        em1=0
        em2=0
        call panbno(itypma, nbnott)
        nno = nbnott(1) + nbnott(2) + nbnott(3)
!
!       BOUCLE SUR LES NOEUDS DE LA MAILLE
        lstch = .false.
        do 311 in = 1, nno
            if(in.gt.1) nunop = nuno
            nuno = connex(zi(jconx2+nmaabs-1)+in-1)
            en = stano(nuno)
            if (en .eq. 1 .or. en .eq. 3) em1=em1+1
            if (en .eq. 2 .or. en .eq. 3) em2=em2+1
            if(in.gt.1.and.cnslt(3:8).eq.'OP0010') lstch = &
            lstch.or.(( zr(jltsv-1+nuno)*zr(jltsv-1+nunop)).le.0.d0)
311     continue
        if (em1 .ge. 1) em=1
        if (em2 .ge. 1) em=2
        if (em1 .ge. 1 .and. em2 .ge. 1) em=3
!
!         MAILLE RETENUE POUR MAFOND (TS LS NOEUDS SONT 'CARRÉS')
!         SOUS RÉSERVE QUE CE SOIT UNE MAILLE PRINCIPALE DE MAFIS
        if(typdis.ne.'COHESIF') then
            if (em2 .eq. nno) then
!
            call dismoi('DIM_TOPO', typma, 'TYPE_MAILLE', repi=dime_topo)
            if (dime_topo .eq. ndim) then
!
                do 312 imae = 1, nmafis
                    if (nmaabs .eq. mafis(imae)) then
                        i=i+1
                        ASSERT(i.le.nmafis)
                        mafon(i)=nmaabs
!                       ON SORT DE LA BOUCLE 312
                        goto 313
                    endif
312             continue
313         continue
            endif
            endif
        else if(typdis.eq.'COHESIF') then
!
            if(lstch.or.(em1.lt.nno.and.em1.gt.0)&
               .and.typma(1:3).ne.'SEG') then
                do 412 imae=1,nmafis
                    if (nmaabs .eq. mafis(imae)) then
                        i=i+1
                        ASSERT(i.le.nmafis)
                        mafon(i)=nmaabs
                        goto 413
                    endif
412             continue
413             continue
            endif
         endif

!       ON RÉCUPÈRE LES NUMEROS DES MAILLES ENRICHIES
        if (em .eq. 1) then
            im1=im1+1
            ASSERT(im1.le.nbma)
            maen1(im1)=nmaabs
        else if (em.eq.2) then
            im2=im2+1
            ASSERT(im2.le.nbma)
            maen2(im2)=nmaabs
        else if (em.eq.3) then
            im3=im3+1
            ASSERT(im3.le.nbma)
            maen3(im3)=nmaabs
        endif
310 end do
!
    nmafon=i
    nmaen1=im1
    nmaen2=im2
    nmaen3=im3
!
    call jedema()
end subroutine
