subroutine comp_meca_mod(keywordfact, iocc, model, ndime_model, nom_mod_mfront)
!
    implicit none
!
#include "jeveux.h"
#include "asterfort/assert.h"
#include "asterfort/dismoi.h"
#include "asterfort/getvtx.h"
#include "asterfort/jedema.h"
#include "asterfort/jemarq.h"
#include "asterfort/jenuno.h"
#include "asterfort/jeveuo.h"
#include "asterfort/jexnum.h"
#include "asterfort/reliem.h"
#include "asterfort/teattr.h"
#include "asterfort/utmess.h"

!
! ======================================================================
! COPYRIGHT (C) 2091 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!   1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
    character(len=16), intent(in) :: keywordfact
    integer :: iocc, ndime_model
    character(len=8), intent(in) :: model
    character(len=16), intent(out) :: nom_mod_mfront
!
! --------------------------------------------------------------------------------------------------
!
! <CARTE> COMPOR
!
! Recherche de l'hypothese de modelisation associe a une occurence
! du mot cle facteur keywordfact
!
! --------------------------------------------------------------------------------------------------
!
! In  keywordfact    : nom du mot cle facteur
! In  iocc           : occurence du mot cle facteur
! In  model          : nom du modele
! In  mesh           : nom du maillage
! Out nom_mod_mfront : type de modelisation
!
! --------------------------------------------------------------------------------------------------
! person_in_charge: nicolas.sellenet at edf.fr
!
    integer :: nt, nb_elem_affe, nb_elem, j_elem_affe, ielem, nume_elem
    integer :: nutyel, iret
    logical :: l_affe_all
    character(len=24) :: list_elem_affe
    character(len=16) :: motcle(2), notype, type_elem, type_elem_save,principal
    character(len=8) :: typmcl(2), mesh
    character(len=1) :: d1
    integer, pointer :: maille(:) => null()

    call jemarq()
!
    call jeveuo(model//'.MAILLE', 'L', vi=maille)
    call dismoi('NOM_MAILLA', model, 'MODELE', repk=mesh)
!
    list_elem_affe = '&&COMPMECASAVE.LIST'
    motcle(1)   = 'GROUP_MA'
    motcle(2)   = 'MAILLE'
    typmcl(1)   = 'GROUP_MA'
    typmcl(2)   = 'MAILLE'
!
    call getvtx(keywordfact, 'TOUT', iocc = iocc, nbret = nt)
    if (nt .ne. 0) then
        l_affe_all = .true.
    else
        l_affe_all = .false.
        call reliem(' ', mesh, 'NU_MAILLE', keywordfact, iocc, &
                    2, motcle, typmcl, list_elem_affe, nb_elem_affe)
        if (nb_elem_affe .eq. 0) l_affe_all = .true.
    endif
!
    if (l_affe_all) then
        call dismoi('NB_MA_MAILLA', mesh, 'MAILLAGE', repi=nb_elem)
    else
        call jeveuo(list_elem_affe, 'L', j_elem_affe)
        nb_elem = nb_elem_affe
    endif
!
    type_elem_save = ' '
    do ielem = 1, nb_elem
!
! ----- Element courant
!
        if (l_affe_all) then
            nume_elem = ielem
        else
            nume_elem = zi(j_elem_affe-1+ielem)
        endif
!
! --------- Recherche du type d'element
!
        nutyel = maille(nume_elem)
        if (nutyel.ne.0) then
            call jenuno(jexnum('&CATA.TE.NOMTE', nutyel), notype)
!
! --------- Type de modelisation
!
            call teattr('C', 'TYPMOD', type_elem, iret, typel=notype)
            call teattr('C', 'PRINCIPAL', principal, iret, typel=notype)
            call teattr('C', 'DIM_TOPO_MODELI', d1, iret, typel=notype)
            read(d1,'(I1)') ndime_model

            if (principal(1:3) .eq. 'OUI') then
!
                if ( type_elem_save.eq.' ' ) type_elem_save = type_elem

                if (type_elem_save.ne.type_elem) then
                  call utmess('F','COMPOR4_13', si=iocc, nk=2, &
                              valk=[type_elem_save, type_elem])
                endif
!
                if ( type_elem.eq.'COMP3D' ) then
                    nom_mod_mfront = '_Tridimensional'
                elseif ( type_elem.eq.'C_PLAN' ) then
                    nom_mod_mfront = '_PlaneStress'
                elseif ( type_elem.eq.'D_PLAN' ) then
                    nom_mod_mfront = '_PlaneStrain'
                elseif ( type_elem.eq.'AXIS' ) then
                    nom_mod_mfront = '_Axisymmetrical'
                elseif ( type_elem.eq.'NON_DEFINI' ) then
! dans phenomene_modelisation.cata, il manque l'attribut TYPMOD pour la THM
                    nom_mod_mfront = '_Tridimensional'
                else
                    ASSERT(.false.)
                endif
            endif

        endif
!
    enddo
!
    call jedema()
!
end subroutine
