function spect2(a, b, xlc, vitn, rhoe,&
                defm, func, tol, ier, r1,&
                err, nbp, im, jm)
!------------------------------------------------------------------
! ======================================================================
! COPYRIGHT (C) 1991 - 2015  EDF R&D                  WWW.CODE-ASTER.ORG
! THIS PROGRAM IS FREE SOFTWARE; YOU CAN REDISTRIBUTE IT AND/OR MODIFY
! IT UNDER THE TERMS OF THE GNU GENERAL PUBLIC LICENSE AS PUBLISHED BY
! THE FREE SOFTWARE FOUNDATION; EITHER VERSION 2 OF THE LICENSE, OR
! (AT YOUR OPTION) ANY LATER VERSION.
!
! THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
! WITHOUT ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF
! MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. SEE THE GNU
! GENERAL PUBLIC LICENSE FOR MORE DETAILS.
!
! YOU SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE
! ALONG WITH THIS PROGRAM; IF NOT, WRITE TO EDF R&D CODE_ASTER,
!    1 AVENUE DU GENERAL DE GAULLE, 92141 CLAMART CEDEX, FRANCE.
! ======================================================================
!-------------------------------------------------------------------
! aslint: disable=W0307
    implicit none
!                                 B  F2(X)
! DESCRIPTION : CALCULE DINTEG = S  S func(X,Y) DY   DX
! -----------                     A  F1(X)
!
!               (S EST LE SYMBOLE DE L'INTEGRALE).
!
!               TOL DONNE LE SEUIL DE CONVERGENCE RELATIVE.
!
!               func EST LA FONCTION A INTEGRER.
!               ELLE DOIT ETRE DECLAREE EXTERNAL DANS L'APPELANT.
!               SA SPECIFICATION EST :
!                        DOUBLE PRECISION FUNCTION func ( X, Y )
!                        DOUBLE PRECISION X, Y
!               L'INTEGRALE INTERNE EST EVALUEE PAR LA FONCTION DINTG2.
!               C'EST DONC DINTG2 QUI APPELLE func ET NON DINTEG.
!               func EST DONNEE EN ARGUMENT A DINTG2 PAR DINTEG.
!
!               A ET B DONNENT LES BORNES DE L'INTEGRALE EXTERNE.
!
!               F1 ET F2 DONNENT LES BORNES DE L'INTEGRALE INTERNE.
!               ELLES DOIVENT ETRE DECLAREE EXTERNAL DANS L'APPELANT.
!               LEURS SPECIFICATIONS SONT :
!                        DOUBLE PRECISION FUNCTION F1 ( X )
!                        DOUBLE PRECISION X
!                        DOUBLE PRECISION FUNCTION F2 ( X )
!                        DOUBLE PRECISION X
!
! AU RETOUR : IER=0 <==> L'INTEGRALE EXTERNE A CONVERGEE.
! ---------   IER=1 <==> L'INTEGRALE EXTERNE N'A PAS CONVERGEE.
!                        ON RENVOIT LA VALEUR AU PAS PRECEDENT ET
!                        L'ERREUR RELATIVE.
!             A, B ET TOL SONT INCHANGES.
!
! *****************   DECLARATIONS DES VARIABLES   ********************
!
!
! ARGUMENTS
! ---------
#include "jeveux.h"
#include "asterfort/spect3.h"
    integer :: nbp
    real(kind=8) :: a
    real(kind=8) :: b
    real(kind=8) :: xlc
    real(kind=8) :: vitn(nbp, *)
    real(kind=8) :: rhoe(nbp, *)
    real(kind=8) :: defm(nbp, *)
    real(kind=8) :: tol
    integer :: ier
    real(kind=8) :: r1
    real(kind=8) :: err
    integer :: im
    integer :: jm
    real(kind=8) :: spect2
    interface
        function func(xx, y, xlc, vitn, rhoe,&
                      defm, nbp, im, jm)
            integer :: nbp
            real(kind=8) :: xx
            real(kind=8) :: y
            real(kind=8) :: xlc
            real(kind=8) :: vitn(nbp, *)
            real(kind=8) :: rhoe(nbp, *)
            real(kind=8) :: defm(nbp, *)
            integer :: im
            integer :: jm
            real(kind=8) :: func
        end function func
    end interface
!
! VARIABLES LOCALES
! -----------------
    integer :: index, n1, n2, i, arret
    real(kind=8) :: res, xm, dx, x0, som, x
    real(kind=8) :: w(127), coeff(381)
!
! DATAS
!-----------------------------------------------------------------------
    data (coeff(i),i=  1, 20) /&
     &     0.7745966692414834d0  ,    0.5555555555555556d0  ,&
     &     0.8888888888888889d0  ,    0.2684880898683334d0  ,&
     &     0.9604912687080203d0  ,    0.1046562260264673d0  ,&
     &     0.4342437493468026d0  ,    0.4013974147759622d0  ,&
     &     0.4509165386584741d0  ,    0.1344152552437842d0  ,&
     &     0.5160328299707974d-1 ,    0.2006285293769890d0  ,&
     &     0.9938319632127550d0  ,    0.1700171962994026d-1 ,&
     &     0.8884592328722570d0  ,    0.9292719531512454d-1 ,&
     &     0.6211029467372264d0  ,    0.1715119091363914d0  ,&
     &     0.2233866864289669d0  ,    0.2191568584015875d0  /
!
    data (coeff(i),i= 21, 40) /&
     &     0.2255104997982067d0  ,    0.6720775429599070d-1 ,&
     &     0.2580759809617665d-1 ,    0.1003142786117956d0  ,&
     &     0.8434565739321106d-2 ,    0.4646289326175799d-1 ,&
     &     0.8575592004999035d-1 ,    0.1095784210559246d0  ,&
     &     0.9990981249676676d0  ,    0.2544780791561874d-2 ,&
     &     0.9815311495537401d0  ,    0.1644604985438781d-1 ,&
     &     0.9296548574297401d0  ,    0.3595710330712932d-1 ,&
     &     0.8367259381688687d0  ,    0.5697950949412336d-1 ,&
     &     0.7024962064915271d0  ,    0.7687962049900353d-1 ,&
     &     0.5313197436443756d0  ,    0.9362710998126447d-1 /
!
    data (coeff(i),i= 41, 60) /&
     &     0.3311353932579768d0  ,    0.1056698935802348d0  ,&
     &     0.1124889431331866d0  ,    0.1119568730209535d0  ,&
     &     0.1127552567207687d0  ,    0.3360387714820773d-1 ,&
     &     0.1290380010035127d-1 ,    0.5015713930589954d-1 ,&
     &     0.4217630441558855d-2 ,    0.2323144663991027d-1 ,&
     &     0.4287796002500773d-1 ,    0.5478921052796287d-1 ,&
     &     0.1265156556230068d-2 ,    0.8223007957235930d-2 ,&
     &     0.1797855156812827d-1 ,    0.2848975474583355d-1 ,&
     &     0.3843981024945553d-1 ,    0.4681355499062801d-1 ,&
     &     0.5283494679011652d-1 ,    0.5597843651047632d-1 /
!
    data (coeff(i),i= 61, 80) /&
     &     0.9998728881203576d0  ,    0.3632214818455307d-3 ,&
     &     0.9972062593722220d0  ,    0.2579049794685688d-2 ,&
     &     0.9886847575474295d0  ,    0.6115506822117246d-2 ,&
     &     0.9721828747485818d0  ,    0.1049824690962132d-1 ,&
     &     0.9463428583734029d0  ,    0.1540675046655950d-1 ,&
     &     0.9103711569570043d0  ,    0.2059423391591271d-1 ,&
     &     0.8639079381936905d0  ,    0.2586967932721475d-1 ,&
     &     0.8069405319502176d0  ,    0.3107355111168796d-1 ,&
     &     0.7397560443526948d0  ,    0.3606443278078257d-1 ,&
     &     0.6629096600247806d0  ,    0.4071551011694432d-1 /
!
    data (coeff(i),i= 81,100) /&
     &     0.5771957100520458d0  ,    0.4491453165363220d-1 ,&
     &     0.4836180269458410d0  ,    0.4856433040667320d-1 ,&
     &     0.3833593241987303d0  ,    0.5158325395204846d-1 ,&
     &     0.2777498220218243d0  ,    0.5390549933526606d-1 ,&
     &     0.1682352515522075d0  ,    0.5548140435655936d-1 ,&
     &     0.5634431304659279d-1 ,    0.5627769983125430d-1 ,&
     &     0.5637762836038472d-1 ,    0.1680193857410387d-1 ,&
     &     0.6451900050175737d-2 ,    0.2507856965294977d-1 ,&
     &     0.2108815245726633d-2 ,    0.1161572331995513d-1 ,&
     &     0.2143898001250387d-1 ,    0.2739460526398143d-1 /
!
    data (coeff(i),i=101,120) /&
     &     0.6326073193626335d-3 ,    0.4111503978654693d-2 ,&
     &     0.8989275784064136d-2 ,    0.1424487737291677d-1 ,&
     &     0.1921990512472777d-1 ,    0.2340677749531401d-1 ,&
     &     0.2641747339505826d-1 ,    0.2798921825523816d-1 ,&
     &     0.1807395644453884d-3 ,    0.1289524082610417d-2 ,&
     &     0.3057753410175531d-2 ,    0.5249123454808859d-2 ,&
     &     0.7703375233279742d-2 ,    0.1029711695795636d-1 ,&
     &     0.1293483966360737d-1 ,    0.1553677555584398d-1 ,&
     &     0.1803221639039129d-1 ,    0.2035775505847216d-1 ,&
     &     0.2245726582681610d-1 ,    0.2428216520333660d-1 /
!
    data (coeff(i),i=121,140) /&
     &     0.2579162697602423d-1 ,    0.2695274966763303d-1 ,&
     &     0.2774070217827968d-1 ,    0.2813884991562715d-1 ,&
     &     0.9999824303548916d0  ,    0.5053609520786252d-4 ,&
     &     0.9995987996719107d0  ,    0.3777466463269847d-3 ,&
     &     0.9983166353184074d0  ,    0.9383698485423815d-3 ,&
     &     0.9957241046984072d0  ,    0.1681142865421470d-2 ,&
     &     0.9914957211781061d0  ,    0.2568764943794020d-2 ,&
     &     0.9853714995985204d0  ,    0.3572892783517300d-2 ,&
     &     0.9771415146397057d0  ,    0.4671050372114322d-2 ,&
     &     0.9666378515584166d0  ,    0.5843449875835640d-2 /
!
    data (coeff(i),i=141,160) /&
     &     0.9537300064257611d0  ,    0.7072489995433555d-2 ,&
     &     0.9383203977795929d0  ,    0.8342838753968158d-2 ,&
     &     0.9203400254700124d0  ,    0.9641177729702537d-2 ,&
     &     0.8997448997769400d0  ,    0.1095573338783790d-1 ,&
     &     0.8765134144847053d0  ,    0.1227583056008277d-1 ,&
     &     0.8506444947683503d0  ,    0.1359157100976555d-1 ,&
     &     0.8221562543649804d0  ,    0.1489364166481518d-1 ,&
     &     0.7910849337998484d0  ,    0.1617321872957772d-1 ,&
     &     0.7574839663805136d0  ,    0.1742193015946417d-1 ,&
     &     0.7214230853700989d0  ,    0.1863184825613879d-1 /
!
    data (coeff(i),i=161,180) /&
     &     0.6829874310910792d0  ,    0.1979549504809750d-1 ,&
     &     0.6422766425097595d0  ,    0.2090585144581202d-1 ,&
     &     0.5994039302422429d0  ,    0.2195636630531782d-1 ,&
     &     0.5544951326319325d0  ,    0.2294096422938775d-1 ,&
     &     0.5076877575337166d0  ,    0.2385405210603854d-1 ,&
     &     0.4591300119898323d0  ,    0.2469052474448768d-1 ,&
     &     0.4089798212298887d0  ,    0.2544576996546477d-1 ,&
     &     0.3574038378315322d0  ,    0.2611567337670610d-1 ,&
     &     0.3045764415567140d0  ,    0.2669662292745036d-1 ,&
     &     0.2506787303034832d0  ,    0.2718551322962479d-1 /
!
    data (coeff(i),i=181,200) /&
     &     0.1958975027111002d0  ,    0.2757974956648187d-1 ,&
     &     0.1404242331525602d0  ,    0.2787725147661370d-1 ,&
     &     0.8445404008371088d-1 ,    0.2807645579381725d-1 ,&
     &     0.2818464894974569d-1 ,    0.2817631903301660d-1 ,&
     &     0.2818881418019236d-1 ,    0.8400969287051933d-2 ,&
     &     0.3225950025087868d-2 ,    0.1253928482647488d-1 ,&
     &     0.1054407622853764d-2 ,    0.5807861659977567d-2 ,&
     &     0.1071949000625193d-1 ,    0.1369730263199072d-1 ,&
     &     0.3163038905577009d-3 ,    0.2055751989327344d-2 ,&
     &     0.4494637892032068d-2 ,    0.7122438686458387d-2 /
!
    data (coeff(i),i=201,220) /&
     &     0.9609952562363883d-2 ,    0.1170338874765700d-1 ,&
     &     0.1320873669752913d-1 ,    0.1399460912761908d-1 ,&
     &     0.9059242831838712d-4 ,    0.6447620408298260d-3 ,&
     &     0.1528876705087616d-2 ,    0.2624561727404430d-2 ,&
     &     0.3851687616639871d-2 ,    0.5148558478978178d-2 ,&
     &     0.6467419831803687d-2 ,    0.7768387777921991d-2 ,&
     &     0.9016108195195643d-2 ,    0.1017887752923608d-1 ,&
     &     0.1122863291340805d-1 ,    0.1214108260166830d-1 ,&
     &     0.1289581348801211d-1 ,    0.1347637483381652d-1 ,&
     &     0.1387035108913984d-1 ,    0.1406942495781358d-1 /
!
    data (coeff(i),i=221,240) /&
     &     0.2234303854547696d-4 ,    0.1888639812523945d-3 ,&
     &     0.4691849097155527d-3 ,    0.8405714326197546d-3 ,&
     &     0.1284382471895813d-2 ,    0.1786446391758630d-2 ,&
     &     0.2335525186057160d-2 ,    0.2921724937917820d-2 ,&
     &     0.3536244997716778d-2 ,    0.4171419376984079d-2 ,&
     &     0.4820588864851268d-2 ,    0.5477866693918951d-2 ,&
     &     0.6137915280041385d-2 ,    0.6795785504882773d-2 ,&
     &     0.7446820832407591d-2 ,    0.8086609364788860d-2 ,&
     &     0.8710965079732087d-2 ,    0.9315924128069395d-2 ,&
     &     0.9897747524048750d-2 ,    0.1045292572290601d-1 /
!
    data (coeff(i),i=241,260) /&
     &     0.1097818315265891d-1 ,    0.1147048211469387d-1 ,&
     &     0.1192702605301927d-1 ,    0.1234526237224384d-1 ,&
     &     0.1272288498273238d-1 ,    0.1305783668835305d-1 ,&
     &     0.1334831146372518d-1 ,    0.1359275661481240d-1 ,&
     &     0.1378987478324094d-1 ,    0.1393862573830685d-1 ,&
     &     0.1403822789690862d-1 ,    0.1408815951650830d-1 ,&
     &     0.9999958664005165d0  ,    0.8822702694609804d-5 ,&
     &     0.9999445736539534d0  ,    0.5403992347935378d-4 ,&
     &     0.9997604612201292d0  ,    0.1357078405902732d-3 ,&
     &     0.9993803390481119d0  ,    0.2492143139346697d-3 /
!
    data (coeff(i),i=261,280) /&
     &     0.9987456144372444d0  ,    0.3897452481631715d-3 ,&
     &     0.9978053544968644d0  ,    0.5542953186526136d-3 ,&
     &     0.9965141459148629d0  ,    0.7402828044420028d-3 ,&
     &     0.9948315028006219d0  ,    0.9453615168885046d-3 ,&
     &     0.9927213442827886d0  ,    0.1167484117433308d-2 ,&
     &     0.9901513704007702d0  ,    0.1404907995655566d-2 ,&
     &     0.9870925279540341d0  ,    0.1656112728154507d-2 ,&
     &     0.9835186575786327d0  ,    0.1919712971013880d-2 ,&
     &     0.9794062816708627d0  ,    0.2194406925363840d-2 ,&
     &     0.9747344597524027d0  ,    0.2478958226657568d-2 /
!
    data (coeff(i),i=281,300) /&
     &     0.9694846595024592d0  ,    0.2772195764593451d-2 ,&
     &     0.9636406215698121d0  ,    0.3073018434702578d-2 ,&
     &     0.9571882161098610d0  ,    0.3380397991086920d-2 ,&
     &     0.9501152975212949d0  ,    0.3693377917025651d-2 ,&
     &     0.9424115651910831d0  ,    0.4011068724075023d-2 ,&
     &     0.9340684361577258d0  ,    0.4332640968092983d-2 ,&
     &     0.9250789329070757d0  ,    0.4657317299756855d-2 ,&
     &     0.9154375871557650d0  ,    0.4984364564765539d-2 ,&
     &     0.9051403588132616d0  ,    0.5313086605187057d-2 ,&
     &     0.8941845683355590d0  ,    0.5642818101384444d-2 /
!
    data (coeff(i),i=301,320) /&
     &     0.8825688402473419d0  ,    0.5972919565508166d-2 ,&
     &     0.8702930555481139d0  ,    0.6302773449085759d-2 ,&
     &     0.8573583108862322d0  ,    0.6631781242901888d-2 ,&
     &     0.8437668826727086d0  ,    0.6959361409390423d-2 ,&
     &     0.8295221946374014d0  ,    0.7284947980553807d-2 ,&
     &     0.8146287876551374d0  ,    0.7607989665719057d-2 ,&
     &     0.7990922909608414d0  ,    0.7927949334294849d-2 ,&
     &     0.7829193941182830d0  ,    0.8244303763032868d-2 ,&
     &     0.7661178193037601d0  ,    0.8556543561307690d-2 ,&
     &     0.7486962936169366d0  ,    0.8864173209482494d-2 /
!
    data (coeff(i),i=321,340) /&
     &     0.7306645212421813d0  ,    0.9166711163560788d-2 ,&
     &     0.7120331553622520d0  ,    0.9463689993830065d-2 ,&
     &     0.6928137697791147d0  ,    0.9754656536317411d-2 ,&
     &     0.6730188302304185d0  ,    0.1003917204405684d-1 ,&
     &     0.6526616654100175d0  ,    0.1031681233094762d-1 ,&
     &     0.6317564377111942d0  ,    0.1058716790488520d-1 ,&
     &     0.6103181137151864d0  ,    0.1084984408933731d-1 ,&
     &     0.5883624344476625d0  ,    0.1110446113400693d-1 ,&
     &     0.5659058854236544d0  ,    0.1135065431598060d-1 ,&
     &     0.5429656664983115d0  ,    0.1158807403304395d-1 /
!
    data (coeff(i),i=341,360) /&
     &     0.5195596615374570d0  ,    0.1181638589083024d-1 ,&
     &     0.4957064079187615d0  ,    0.1203527078527956d-1 ,&
     &     0.4714250658716589d0  ,    0.1224442498161199d-1 ,&
     &     0.4467353876620285d0  ,    0.1244356019071404d-1 ,&
     &     0.4216576866261633d0  ,    0.1263240364354208d-1 ,&
     &     0.3962128060576159d0  ,    0.1281069816387736d-1 ,&
     &     0.3704220879500782d0  ,    0.1297820223953740d-1 ,&
     &     0.3443073415994380d0  ,    0.1313469009196015d-1 ,&
     &     0.3178908120684767d0  ,    0.1327995174393053d-1 ,&
     &     0.2911951485182467d0  ,    0.1341379308511010d-1 /
!
    data (coeff(i),i=361,381) /&
     &     0.2642433724109268d0  ,    0.1353603593495621d-1 ,&
     &     0.2370588455898297d0  ,    0.1364651810257129d-1 ,&
     &     0.2096652382431812d0  ,    0.1374509344300190d-1 ,&
     &     0.1820864967592522d0  ,    0.1383163190950643d-1 ,&
     &     0.1543468114813781d0  ,    0.1390601960132546d-1 ,&
     &     0.1264705843723020d0  ,    0.1396815880651694d-1 ,&
     &     0.9848239659811920d-1 ,    0.1401796803945661d-1 ,&
     &     0.7040697604285518d-1 ,    0.1405538207264996d-1 ,&
     &     0.4226916476536360d-1 ,    0.1408035196255366d-1 ,&
     &     0.1409388641078246d-1 ,    0.1409284506916041d-1 ,&
     &     0.1409440709009618d-1 /
!
! *****************    DEBUT DU CODE EXECUTABLE    *********************
!
    ier = 0
    err = 0.d0
    res = 0.0d0
    if (abs(a-b) .lt. 1.0d-30) then
        spect2 = res
        goto 9999
    endif
!
    xm = ( a + b ) / 2.0d0
    dx = ( b - a ) / 2.0d0
    x0 = spect3 ( xm, a,b, func, tol, coeff,xlc,vitn,defm, rhoe,nbp,im,jm )
    r1 = (x0+x0) * dx
    index = 0
    n1 = 0
    n2 = 1
    som = 0.0d0
!
! --- REPETER ...
!
10  continue
    n1 = n1 + n2
    do 20 i = n2, n1
        index = index + 1
        x = coeff(index) * dx
        w(i) = spect3( xm+x, a, b, func, tol, coeff,xlc,vitn, defm,rhoe, nbp,im,jm ) + spect3(xm-x,&
               & a, b, func, tol, coeff,xlc,vitn, defm,rhoe,nbp,im,jm )
        index = index + 1
        som = som + coeff(index)*w(i)
20  end do
    n2 = n1 + 1
    index = index + 1
    res = ( som + coeff(index)*x0 ) * dx
!
! --- TEST DE CONVERGENCE.
!
    if (abs(res-r1) .le. abs(r1*tol)) then
        ier = 0
        arret = 1
    else
        ier = 1
        if (n1 .ge. 127) then
            arret = 1
            err = abs((res-r1)/r1)*100.d0
        else
            arret = 0
            r1 = res
            som = 0.0d0
            do 22 i = 1, n1
                index = index + 1
                som = som + coeff(index)*w(i)
22          continue
        endif
    endif
!
! --- JUSQUE ARRET = 1.
!
    if (arret .eq. 0) goto 10
!
    spect2 = res
!
9999  continue
end function
